--- /Users/Sebastian/Documents/Develop/3ros/Asterisk/1.8-SVN/apps/app_queue.c
+++ /Users/Sebastian/Documents/Develop/3ros/Asterisk/1.8-SVN/apps/app_queue.c
@@ -63,7 +63,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 366880 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include <sys/time.h>
 #include <sys/signal.h>
@@ -910,6 +910,8 @@
 
 static char *app_ql = "QueueLog" ;
 
+static char *app_qupd = "QueueUpdate";
+
 /*! \brief Persistent Members astdb family */
 static const char * const pm_family = "Queue/PersistentMembers";
 /* The maximum length of each persistent member queue database entry */
@@ -5763,6 +5765,84 @@
 	return 0;
 }
 
+
+static int qupd_exec(struct ast_channel *chan, const char *data)
+{
+	int oldtalktime;
+	char *parse;
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(queuename);
+		AST_APP_ARG(uniqueid);
+		AST_APP_ARG(membername);
+		AST_APP_ARG(status);
+		AST_APP_ARG(talktime);
+		AST_APP_ARG(params);
+	);
+
+	if (ast_strlen_zero(data)) {
+		ast_log(LOG_WARNING, "UpdateQueue requires arguments (queuename,uniqueid,membername,status,talktime,params[totaltime,callednumber])\n");
+		return -1;
+	}
+
+	parse = ast_strdupa(data);
+
+	AST_STANDARD_APP_ARGS(args, parse);
+
+	if (ast_strlen_zero(args.queuename) || ast_strlen_zero(args.uniqueid) || ast_strlen_zero(args.membername)|| ast_strlen_zero(args.status)) {
+		ast_log(LOG_WARNING, "Missing argument to UpdateQueue (queuename,uniqueid,membername,status,talktime,params[totaltime,callednumber])\n");
+		return -1;
+	}
+
+	struct call_queue *q;
+	struct member *mem;
+	int newtalktime = 0;
+	
+	if (!ast_strlen_zero(args.talktime))
+	{
+		newtalktime = atoi(args.talktime);
+	}
+
+	q = load_realtime_queue(args.queuename);
+  
+        if (q != NULL) {	
+			 ao2_lock(q); 
+			 if (q->members) {	
+					struct ao2_iterator mem_iter = ao2_iterator_init(q->members, 0);
+					while ((mem = ao2_iterator_next(&mem_iter))) {
+						if (!strcasecmp(mem->membername, args.membername)) 
+							{														 
+							 if (!strcasecmp(args.status, "ANSWER"))
+							 {
+							 	oldtalktime = q->talktime;
+							 	q->talktime = (((oldtalktime << 2) - oldtalktime) + newtalktime) >> 2;
+							 	time(&mem->lastcall);
+								mem->calls++;
+								mem->lastqueue = q;
+							 	q->callscompleted++;
+							 		
+							 		if (newtalktime <= q->servicelevel)
+							 		{
+							 			q->callscompletedinsl++;	
+							 		}		
+							 }
+							 else
+							 {
+							 		q->callsabandoned++;
+							 }
+							 						
+							 ast_queue_log(args.queuename, args.uniqueid, args.membername, "OUTCALL", "%s|%s|%s", args.status, args.talktime, args.params);
+			
+							}
+						ao2_ref(mem, -1);
+					}
+					ao2_iterator_destroy(&mem_iter);
+			}
+			ao2_unlock(q);
+			queue_t_unref(q, "Done with temporary pointer");
+	}
+	return 0;
+}
+
 /*! \brief Copy rule from global list into specified queue */
 static void copy_rules(struct queue_ent *qe, const char *rulename)
 {
@@ -8290,11 +8370,13 @@
 	res |= ast_manager_unregister("QueueRemove");
 	res |= ast_manager_unregister("QueuePause");
 	res |= ast_manager_unregister("QueueLog");
+        res |= ast_manager_unregister("QueueUpdate");
 	res |= ast_manager_unregister("QueuePenalty");
 	res |= ast_unregister_application(app_aqm);
 	res |= ast_unregister_application(app_rqm);
 	res |= ast_unregister_application(app_pqm);
 	res |= ast_unregister_application(app_upqm);
+        res |= ast_unregister_application(app_qupd);
 	res |= ast_unregister_application(app_ql);
 	res |= ast_unregister_application(app);
 	res |= ast_custom_function_unregister(&queueexists_function);
@@ -8360,6 +8442,7 @@
 	res |= ast_register_application_xml(app_pqm, pqm_exec);
 	res |= ast_register_application_xml(app_upqm, upqm_exec);
 	res |= ast_register_application_xml(app_ql, ql_exec);
+        res |= ast_register_application_xml(app_qupd, qupd_exec);
 	res |= ast_manager_register_xml("Queues", 0, manager_queues_show);
 	res |= ast_manager_register_xml("QueueStatus", 0, manager_queues_status);
 	res |= ast_manager_register_xml("QueueSummary", 0, manager_queues_summary);
